# init.py
# This module contains initial conditions generating functions

import os
import numpy as np

#====================== InitPositionsLattice =============================
def InitPositionsLattice(N, L):
    """Returns an array of initial positions of each atom,
placed on a cubic lattice for convenience.
Input:
    N: number of atoms
    L: box length
Output:
    Pos: (N,3) array of positions
"""
    #make the position array
    Pos = np.zeros((N,3), float)
    #compute integer grid # of locations for cubic lattice
    NLat = int(N**(1./3.) + 1.)
    LatSpac = L / NLat
    #make an array of lattice sites
    r = LatSpac * np.arange(NLat, dtype=float) - 0.5*L
    #loop through x, y, z positions in lattice until done
    #for every atom in the system
    i = 0
    for x in r:
        for y in r:
            for z in r:
                Pos[i] = np.array([x,y,z], float)
                #add a random offset to help initial minimization
                Offset = 0.1 * LatSpac * (np.random.rand(3) - 0.5)
                Pos[i] = Pos[i] + Offset
                i += 1
                #if done placing atoms, return
                if i >= N:
                    return Pos
    return Pos
#======================================================================================




#=========================== InitPositionsRandom ================================
def InitPositionsRandom(N, L):
	"""Returns an array of initial positions of each atom,
placed randomly within a box of dimensions L.
Input:
    N: number of atoms
    L: box width
Output:
    Pos: (N,3) array of positions"""
	Pos = L*np.random.rand(N,3)
	return Pos
	
	
	
#============================== Center ====================================
def Center(Pos, N):
	"""Shifts all positions in the array to make center of mass at origin.
Input:
	Pos: (N,3) array of positions
Output
	Pos: (N,3) array of centered positions"""
	CM = sum(Pos)*1./N
	Pos = Pos - CM
	return Pos
	
def CenterP(Pos, Np, Ns):
	"""Shifts all positions in the array to make the center of mass of the POLYMER at the origin.
Input:
	Pos: (Np+Ns,3) array of positions
Output:
	Pos: (Np+Ns,3) array of shifted positions"""
	PosP = Pos[0:Np-1,:]
	CM = sum(PosP)*1./Np
	Pos = Pos - CM
	return Pos


#============================== Relax =======================================
def Relax(Pos, Np, Ns, L, IfRelaxed):
	"""Makes sure none of the S-atoms are too close to the P-atoms.
Input:
	Pos: (N,3) array of positions, P-positions then S-positions
	Np: number of P-atoms
	Ns: number of N-atoms
Output:
	Pos: (N,3) array of relaxed positions"""
	for i in range(Np, Np+Ns-1):
		for j in range(0, i-1):
			r = Pos[j] - Pos[i]
			# take the dot product of the vector r with itself
			d2 = np.dot(r,r)
			d = np.sqrt(d2)
			if d < 2.5:
				Pos[i] = L*np.random.rand(1,3)
				# Use the following for ex13, comment otherwise
				Pos[i,0] = Pos[i,0] - L/2.
				Pos[i,2] = Pos[i,2] - L/2.
				IfRelaxed = False
		for j in range(i+1, Np+Ns-1):
			r = Pos[j] - Pos[i]
			# take the dot product of the vector r with itself
			d2 = np.dot(r,r)
			d = np.sqrt(d2)
			if d < 2.5:
				Pos[i] = L*np.random.rand(1,3)
				Pos[i,0] = Pos[i,0] - L/2.
				Pos[i,2] = Pos[i,2] - L/2.
				IfRelaxed = False
	return Pos, IfRelaxed



#=========================== RescaleVelocities ================================
def RescaleVelocities(Vel, T):
    """Rescales velocities in the system to the target temperature.
Input:
    Vel: (N,3) array of atomic velocities
    T: target temperature
Output:
    Vel: same as above
"""
    #recenter to zero net momentum (assuming all masses same)
    Vel = Vel - Vel.mean(axis=0)
    #find the total kinetic energy
    KE = 0.5 * np.sum(Vel * Vel)
    #find velocity scale factor from ratios of kinetic energy
    VScale = np.sqrt(1.5 * len(Vel) * T / KE)
    Vel = Vel * VScale
    return Vel  




#=========================== InitVelocities ================================
def InitVelocities(N, T):
    """Returns an initial random velocity set.
Input:
    N: number of atoms
    T: target temperature
Output:
    Vel: (N,3) array of atomic velocities
"""
    Vel = np.random.rand(N, 3)
    Vel = RescaleVelocities(Vel, T)
    return Vel




#=========================== InitAccel ================================
def InitAccel(Pos):
    """Returns the initial acceleration array.
Input:
    Pos: (N,3) array of atomic positions
Output:
    Accel: (N,3) array of acceleration vectors
"""
    #get the acceleration from the forces
    PEnergy, Accel = CalcEnergyForces(Pos, np.zeros_like(Pos))
    return Accel



#=========================== InstTemp ================================
def InstTemp(Vel):
    """Returns the instantaneous temperature.
Input:
    Vel: (N,3) array of atomic velocities
Output:
    Tinst: float
"""
    return np.sum(Vel * Vel) / (3. * len(Vel))