! to compile:
! f2py –c –m MODULENAME SOURCE.f90 --fcompiler=gnu95

! P-P: Harmonic for neighbors, LJ for others
! P-S: LJ
! S-S: LJ OR soft-core repulsion depending on commenting

! Forces include F(U) + F(Visc) + F(Random) + F(Pull) i.e. Lengevin Dynamics with Pulling


!======================================================================================================
!                                   random_normal
!======================================================================================================
subroutine random_normal(Eta)
    real(8), intent(out) :: Eta
    real(8) :: s = 0.44871, t = -0.386595, a = 0.19600, b = 0.25472, r1 = 0.27597, r2 = 0.27846, u, v, x, y, q
	
    do
        call random_number(u)
        call random_number(v)
        v = 1.7156 * (v - 0.5)
        x = u - s
        y = abs(v) - t
        q = x**2 + y*(a*y - b*x)
        
        if (q < r1) exit
!
        if (q > r2) cycle
!
        if (v**2 < -4.0*log(u)*u**2) exit
    end do
    Eta = v/u
end subroutine


!========================================================================================================
!==============================      CalcEnergyForces          ==========================================
!========================================================================================================


subroutine CalcEnergyForces(Pos, Vel, Np, Ns, L, rc, EpsPP, EpsSS, EpsSP, SigPP, SigSS, SigSP, T, dt, PEnergy, Forces, Dim)
    implicit none
    integer, intent(in) :: Np, Ns, Dim
    real(8), intent(in) :: L, rc, EpsPP, EpsSS, EpsSP, SigPP, SigSS, SigSP, T, dt
    real(8), intent(out) :: PEnergy
    real(8), intent(in), dimension(0:Np+Ns-1, 0:Dim-1) :: Pos, Vel
    real(8), intent(inout), dimension(0:Np+Ns-1, 0:Dim-1) :: Forces
!f2py intent(in, out, inplace) :: Forces

    real(8), dimension(Dim) :: rij, Fij, Posi
! k = spring constant for polymer bonds
! r0 = equilibrium distance for polymer bonds
! gamma = viscosity
    real(8), parameter :: k = 3000., r0 = 1., gamma = 10., Fpull = 400.
    real(8) :: d1, d2, id2, id6, id12, y
    real(8) :: rc2, ShiftPP, ShiftSS, ShiftSP, alpha
    integer :: i, j, NAtom
    real(8) :: EtaX, EtaY, EtaZ, Leng
    
    integer :: count, SeedSize
    integer, dimension(1) :: seed

    PEnergy = 0.
    Forces = 0.
    alpha = 0.01 * dble(NAtom)**(-2./3.)
    Leng = sqrt(6. * gamma * T / dt)
    !compute the Lennard-Jones shift value
    ShiftPP = -4. * EpsPP * ((rc/SigPP)**(-12) - (rc/SigPP)**(-6))
    ShiftSS = -4. * EpsSS * ((rc/SigSS)**(-12) - (rc/SigSS)**(-6))
    ShiftSP = -4. * EpsSP * ((rc/SigSP)**(-12) - (rc/SigSP)**(-6))
    rc2 = rc**2
    
    NAtom = Np+Ns
    
    call system_clock(count)
    seed = count
    call random_seed
    call random_seed( size = SeedSize )
    call random_seed( put = seed(1:SeedSize) )
    
    do i = 0, NAtom-1
        !store Pos(i,:) in a temporary array for faster access in j loop
        Posi = Pos(i,:)
        do j = i+1, NAtom-1
            !compute rij, a vector
            rij = Pos(j,:) - Posi
            !compute the minimum image distance (periodic boundary conditions)
            !still a vector
            !rij = rij - L * dnint(rij / L) 
            d2 = sum(rij*rij)
            if (j == i + 1 .and. j < Np) then
                !print *, "first loop"
                !this is a bonded interaction between adjacent atoms in the polymer;
                !use the harmonic bond potential here
                d1 = sqrt(d2) !d1 is the magnitude of rij
                PEnergy = PEnergy + (k/2)*(d1-r0)**2
                Fij = k*(d1-r0)*rij/d1
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
            else if (i < Np .and. j > Np-1) then
                !print *, "second loop"
                !this is a nonbonded interaction between a polymer atom and an Ag atom;
                !use a Lennard-Jones potential here
                if (d2 > rc2) then
                    cycle
                end if
                id2 = SigSP / d2
                id6 = id2 * id2 * id2
                id12 = id6 * id6
                PEnergy = PEnergy + 4. * EpsSP * (id12 - id6) + ShiftSP
                Fij = rij * EpsSP * ((-48. * id12 + 24. * id6) * id2)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
            else if (i < Np-1 .and. i+1 < j .and. j < Np) then
                !print *, "third loop"
			    !this is a nonbonded interaction between two atoms of the polymer;
				!use a Lennard-Jones potential here
                if (d2 > rc2) then
                    cycle
                end if
                id2 = SigPP / d2
                id6 = id2 * id2 * id2
                id12 = id6*id6
                PEnergy = PEnergy + 4. * EpsPP * (id12 - id6) + ShiftPP
                Fij = rij * EpsPP * ((-48. * id12 + 24. * id6) * id2)
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
            else
                !print *, "fourth loop"
                !this is a nonbonded interaction between two Ag atoms;
                !use the Lennard-Jones potential here, with EpsSS and SigSS
                id2 = SigSS / d2
                id6 = id2 * id2 * id2
                id12 = id6 * id6
                PEnergy = PEnergy + 4. * EpsSS * (id12 - id6) + ShiftSS
                !PEnergy = PEnergy + 4. * EpsSS * id12
                Fij = rij * EpsSS * ((-48. * id12 + 24. * id6) * id2)
                !Fij = rij * EpsSS * (-48. * id12) * id2
                Forces(i,:) = Forces(i,:) + Fij
                Forces(j,:) = Forces(j,:) - Fij
           end if
        enddo
        
        call random_normal(EtaX) !produces random number Eta
        call random_normal(EtaY)
        call random_normal(EtaZ)
        !Add the viscous and random forces for Lengevin Dynamics
        Forces(i,0) = Forces(i,0) - gamma * Vel(i,0) + Leng * EtaX
        Forces(i,1) = Forces(i,1) - gamma * Vel(i,1) + Leng * EtaY
        Forces(i,2) = Forces(i,2) - gamma * Vel(i,2) + Leng * EtaZ
        !Add an additional soft-repulsion from the surface (y = 0)
        y = Pos(i,1)
        d2 = y*y
        id2 = 1./d2 !I've plugged in 1 for sigma here
        id6 = id2 * id2 * id2
        id12 = id6 * id6
        Forces(i,1) = Forces(i,1) + 12 * y * id12 * id2
    enddo
!   Set all the forces on one end of the polymer to zero
    Forces(0,0) = 0.
    Forces(0,1) = 0.
    Forces(0,2) = 0.
!   Add an external pulling force to the forces acting on the atom at the other end of the polymer
    Forces(Np-1,1) = Forces(Np-1,1) + Fpull
end subroutine

!=======================================================================================================
!==============================      VVIntegrate             ==========================================
!========================================================================================================

subroutine VVIntegrate(Pos, Vel, Accel, Np, Ns, L, rc, dt, T, EpsPP, EpsSS, EpsSP, SigPP, SigSS, SigSP, Dim, KEnergy, PEnergy)
    implicit none
    integer, intent(in) :: Np, Ns, Dim
    real(8), intent(in) :: L, rc, dt, EpsPP, EpsSS, EpsSP, SigPP, SigSS, SigSP, T
    real(8), intent(inout), dimension(0:Np+Ns-1, 0:Dim-1) :: Pos, Vel, Accel
!f2py intent(in,out,inplace) :: Pos, Vel, Accel
    real(8), intent(out) :: KEnergy, PEnergy
    external :: CalcEnergyForces
    Pos = Pos + dt * Vel + 0.5 * dt*dt * Accel
    Vel = Vel + 0.5 * dt * Accel
    call CalcEnergyForces(Pos, Vel, Np, Ns, L, rc, EpsPP, EpsSS, EpsSP, SigPP, SigSS, SigSP, T, dt, PEnergy, Accel, Dim)
    Vel = Vel + 0.5 * dt * Accel
    KEnergy = 0.5 * sum(Vel*Vel)
end subroutine